
#include <utility/hooks.h>
#include <proto/exec.h>
#include <exec/memory.h>
#include <emul/emulinterface.h>
#include "entries.h"

static ULONG HookStub(void)
{
	struct Hook *hook = (struct Hook *) REG_A0;
	APTR object = (APTR) REG_A2;
	APTR message = (APTR) REG_A1;
	HOOKENTRY func = (HOOKENTRY) hook->h_SubEntry;
	return (*func)(hook->h_Data, object, message);
};

static struct EmulLibEntry HookStubGate =
{
	TRAP_LIB, 0, (void (*)(void)) HookStub
};

LIBAPI void InitHook(struct Hook *hook, HOOKENTRY func, APTR data)
{
	hook->h_Entry = (HOOKFUNC) &HookStubGate;
	hook->h_SubEntry = (HOOKFUNC) func;
	hook->h_Data = data;
}

LIBAPI APTR AllocProcEntry(void (*func)(void))
{
	struct EmulLibEntry *emul = AllocMem(sizeof(struct EmulLibEntry), MEMF_PUBLIC);
	if (emul)
	{
		emul->Trap = TRAP_LIB;
		emul->Extension = 0;
		emul->Func = func;
	}
	return emul;
}

LIBAPI void FreeProcEntry(APTR procEntry)
{
	FreeMem(procEntry, sizeof(struct EmulLibEntry));
}

