
#include "global.h"
#include "audio.h"
#include "ptplay.h"

#include <string.h>
#include <exec/memory.h>
#include <proto/exec.h>

/**************************************************************************
**
**	loadbuf
*/

static BYTE *loadbuf(global *g, STRPTR name, LONG *lenp)
{
	BPTR file;
	
	*lenp = 0;
	file = Open(name, MODE_OLDFILE);
	if (file)
	{
		BYTE *buf = NULL;
		BOOL success = FALSE;
		
		if (Seek(file, 0, OFFSET_END) != -1)
		{
			*lenp = Seek(file, 0, OFFSET_CURRENT);
		}

		if (*lenp > 0)
		{
			Seek(file, 0, OFFSET_BEGINNING);
			buf = AllocMem(*lenp, MEMF_PUBLIC);
			if (buf)
			{
				success = (Read(file, buf, *lenp) == *lenp);
			}
		}

		Close(file);
		
		if (success) return buf;
		if (buf) FreeMem(buf, *lenp);
	}
	
	return NULL;
}

/**************************************************************************
**
**	render
*/

struct PlayData
{
	global *global;
	pt_mod_s *mod;
};

static void render(struct PlayData *play, BYTE *buf1, BYTE *buf2, LONG offs, 
	LONG numsmp, LONG scale, LONG depth, LONG chans)
{
	ULONG flags = play->mod->flags;

	pt_render(play->mod, buf1, buf2, offs, numsmp, scale, depth, chans);

	if (play->global->args[ARG_SONGEND])
	{
		if ((flags ^ play->mod->flags) & MODF_SONGEND)
		{
			play->mod->flags &= ~MODF_SONGEND;
			Signal((struct Task *) play->global->self, SIGBREAKF_CTRL_D);
		}
	}
}

/**************************************************************************
**
**	status = playmod(g, a, modname)
**		0  ... played okay
**		-1 ... aborted with CTRL_C
**		>0 ... DOS error code
*/

static LONG playmod(global *g, audio *a, STRPTR modname)
{
	LONG rc;
	LONG modlen;
	BYTE *modbuf;
	
	rc = ERROR_OBJECT_NOT_FOUND;
	modbuf = loadbuf(g, modname, &modlen);
	if (modbuf)
	{
		pt_mod_s *pmod;

		rc = ERROR_OBJECT_WRONG_TYPE;
		
		pmod = pt_init(modbuf, modlen, a->mixFreq);
		if (pmod)
		{
			ULONG sig;
			struct PlayData pdata;

			pmod->mastervolume = *(LONG *) g->args[ARG_VOLUME] * 256 / 100;
			if (g->args[ARG_NOLED]) pmod->flags &= ~MODF_ALLOWFILTER;

			pdata.global = g;
			pdata.mod = pmod;
			SetAudioFunc(g, a, (FILLFUNC) render, &pdata);
			PlayAudio(g, a, TRUE);
		
			sig = Wait(SIGBREAKF_CTRL_C | SIGBREAKF_CTRL_D);

			PlayAudio(g, a, FALSE);
	
			pt_free(pmod);

			rc = (sig & SIGBREAKF_CTRL_C) ? -1 : 0;
		}
		
		FreeMem(modbuf, modlen);
	}
	
	return rc;
}

/**************************************************************************
**
**	main
*/

int main(int argc, char **argv)
{
	LONG rc = RETURN_FAIL;

	global gdata;
	audio adata;

	global *g = &gdata;
	audio *a = &adata;
	
	if (InitGlobal(g))
	{
		struct RDArgs *ra;
		LONG freq = 44100;
		LONG vol = 100;
		LONG args[ARG_NUM];
		
		memset(args, 0, sizeof(args));
		args[ARG_VOLUME] = (LONG) &vol;
		args[ARG_FREQ] = (LONG) &freq;
		g->args = args;

		ra = ReadArgs(ARG_TEMPLATE, args, NULL);
		if (ra)
		{
			if (InitAudio(g, a))
			{
				rc = RETURN_OK;			

				do
				{
					STRPTR *modlist = (STRPTR *) args[ARG_FROM];
	
					while (*modlist)
					{
						LONG res;
	
						PutStr("Playing ");
						PutStr(*modlist);
						PutStr("\n");
						
						res = playmod(g, a, *modlist);
						if (res == -1)
						{
							PutStr("*** Aborted\n");
							rc = RETURN_WARN;
							break;
						}
						else if (res > 0)
						{
							PutStr("*** ");
							PutStr(*modlist);
							PutStr(" could not be loaded\n");
						}
	
						modlist++;
					}

				} while (g->args[ARG_LOOP] && rc == RETURN_OK);

				ExitAudio(g, a);
			}
			else
			{
				PutStr("*** Could not initialize audio\n");
			}
			
			FreeArgs(ra);
		}
		else
		{
			PutStr("Modplay - written by Ronald Hof, Timm S. Mueller, Per Johansson\n");
			PutStr("Visit our webpage at http://ptplay.k-n-p.org\n\n");
		
			PutStr("FROM/M/A     - Protracker module(s)\n");
			PutStr("SONGEND/S    - Detect song-ends\n");
			PutStr("LOOP/S       - Loop playlist\n");
			PutStr("V=VOLUME/N   - Volume [percent]. Use >100 for boost\n");
			PutStr("F=FREQ/N     - Output frequency [Hertz]\n");
			PutStr("S=STEREO/S   - Use Protracker stereo layout\n");
			PutStr("HIQ/S        - Double oversampling\n");
			PutStr("NOLED/S      - Do not emulate lowpass filter\n");
			PutStr("CTRL-C to abort, CTRL-D to advance to next module\n");
		}
		
		ExitGlobal(g);
	}
	
	return rc;
}

