
#include "tek_global.h"
#include <tek/mod/audio.h>

static TTASKENTRY TBOOL audio_init(TAPTR task);
static TTASKENTRY TVOID audio_main(TAPTR task);

struct Audio *
audio_create(void)
{
	TAPTR task = TExecCreateTask(TExecBase, audio_main, audio_init, TNULL);
	if (task) return TExecGetTaskData(TExecBase, task);
	return TNULL;
}

static TVOID TCALLBACK
audio_destroy(struct Audio *a)
{
	if (a->aud_Task)
	{
		TExecSignal(TExecBase, a->aud_Task, TTASK_SIG_ABORT);
		TDestroy(a->aud_Task);
	}
	TExecFree(TExecBase, a);
}

static TVOID
audio_free(struct Audio *a)
{
	TDestroy(a->aud_PlayLock);

	if (a->ioreq2)
	{
		TExecFree(TExecBase, a->ioreq2->audio_Std.io_Data);
		TExecFree(TExecBase, a->ioreq2);
	}

	if (a->ioreq1)
	{
		TExecCloseModule(TExecBase, a->ioreq1->audio_Std.io_Req.io_Device);
		TDestroy(a->ioreq1->audio_Std.io_Req.io_ReplyPort);
		TExecFree(TExecBase, a->ioreq1->audio_Std.io_Data);
		TExecFree(TExecBase, a->ioreq1);
	}
}

static TTASKENTRY TBOOL
audio_init(TAPTR task)
{
	struct Audio *a = TExecAlloc0(TExecBase, TNULL, sizeof(struct Audio));
	if (a)
	{
		a->aud_PlayLock = TExecCreateLock(TExecBase, TNULL);
		if (a->aud_PlayLock)
		{
			TTAGITEM audiotags[3];
			TUINT prefformats[2];
			TUINT prefrates[2];

			TUINT format = DEFAULT_FORMAT;	//TADIOFMT_S16M;
			TINT rate = DEFAULT_RATE;		//MSS_FREQ;

			prefformats[0] = format;
			prefformats[1] = 0xffffffff;
			prefrates[0] = rate;
			prefrates[1] = 0xffffffff;

			audiotags[0].tti_Tag = TADIOT_PrefFormats;
			audiotags[0].tti_Value = prefformats;
			audiotags[1].tti_Tag = TADIOT_PrefRates;
			audiotags[1].tti_Value = prefrates;
			audiotags[2].tti_Tag = TTAG_DONE;

			a->ioreq1 = TExecOpenModule(TExecBase, "audio", 0, audiotags);
			if (a->ioreq1)
			{
				TINT ioreqsize = TExecGetSize(TExecBase, a->ioreq1);
				a->ioreq2 = TExecAllocMsg(TExecBase, ioreqsize);
				if (a->ioreq2)
				{
					TAPTR replyport = TExecCreatePort(TExecBase, TNULL);
					if (replyport)
					{
						TINT numchn, numbpc, numbps, numsmp;

						numsmp = (TINT) TGetTag(a->ioreq1->audio_Tags, TADIOT_FragmentSize, (TTAG) 1024);
						if (Arguments[ARG_BUFSIZE]) numsmp = *((TINT *) Arguments[ARG_BUFSIZE]);
						numchn = TADIO_GETNUMCHANNELS(format);
						numbpc = TADIO_GETBYTESPERCHAN(format);
						numbps = numchn * numbpc;

						a->aud_Rate = (TINT) TGetTag(a->ioreq1->audio_Tags, TADIOT_MixRate, (TTAG) rate);

						a->ioreq1->audio_Std.io_Length = numbps * numsmp;
						a->ioreq1->audio_Std.io_Req.io_Command = TIOCMD_WRITE;
						a->ioreq1->audio_Std.io_Req.io_ReplyPort = replyport;
						a->ioreq1->audio_Format = format;
						a->ioreq1->audio_Rate = rate;

						TExecCopyMem(TExecBase, a->ioreq1, a->ioreq2, ioreqsize);

						a->ioreq1->audio_Std.io_Data = TExecAlloc0(TExecBase, TNULL, numbps * numsmp);
						a->ioreq2->audio_Std.io_Data = TExecAlloc0(TExecBase, TNULL, numbps * numsmp);

						if (a->ioreq1->audio_Std.io_Data &&
							a->ioreq2->audio_Std.io_Data)
						{
							a->aud_NumSmp = numsmp;
							a->aud_NumChn = numchn;
							a->aud_NumBPC = numbpc;
							a->aud_NumBPS = numbps;

							printf("rate  : %d\n", rate);
							printf("numsmp: %d\n", numsmp);
							printf("numchn: %d\n", numchn);
							printf("numbpc: %d\n", numbpc);
							printf("numbps: %d\n", numbps);

							a->ioreq1->audio_Link = TNULL;
							a->ioreq1->audio_Volume = 0;
							TExecPutIO(TExecBase, (struct TIORequest *) a->ioreq1);

							a->ioreq2->audio_Link = a->ioreq1;
							a->ioreq2->audio_Volume = 0;
							TExecPutIO(TExecBase, (struct TIORequest *) a->ioreq2);

							a->aud_Task = task;
							a->aud_Handle.thn_DestroyFunc = (TDFUNC) audio_destroy;
							TExecSetTaskData(TExecBase, task, a);

							return TTRUE;
						}
					}
				}
			}
			audio_free(a);
		}
		TExecFree(TExecBase, a);
	}

	return TFALSE;
}

static TTASKENTRY TVOID
audio_main(TAPTR task)
{
	struct Audio *a = TExecGetTaskData(TExecBase, task);
	struct TAudIORequest *ioreq1, *ioreq2, *ioreqt;
	TINT vol = 0x10000;
	TUINT sig;

	ioreq1 = a->ioreq1;
	ioreq2 = a->ioreq2;

	do
	{
		TExecWaitIO(TExecBase, (struct TIORequest *) ioreq1);

		TExecLock(TExecBase, a->aud_PlayLock);
		if (a->aud_PlayFunc) (*a->aud_PlayFunc)(a, ioreq1->audio_Std.io_Data);
		TExecUnlock(TExecBase, a->aud_PlayLock);

		ioreq1->audio_Link = ioreq2;
		ioreq1->audio_Volume = vol;
		TExecPutIO(TExecBase, (struct TIORequest *) ioreq1);

		ioreqt = ioreq1; ioreq1 = ioreq2; ioreq2 = ioreqt;

		sig = TExecSetSignal(TExecBase, 0, TTASK_SIG_ABORT);

	} while (!(sig & TTASK_SIG_ABORT));

	TExecAbortIO(TExecBase, (struct TIORequest *) ioreq1);
	TExecAbortIO(TExecBase, (struct TIORequest *) ioreq2);

	TExecWaitIO(TExecBase, (struct TIORequest *) ioreq1);
	TExecWaitIO(TExecBase, (struct TIORequest *) ioreq2);

	audio_free(a);
}

TVOID
audio_setplayfunc(struct Audio *a, PLAYFUNC f)
{
	TExecLock(TExecBase, a->aud_PlayLock);
	a->aud_PlayFunc = f;
	TExecUnlock(TExecBase, a->aud_PlayLock);
}
