
#include "tek_global.h"

static TVOID TCALLBACK
disp_destroy(struct Display *d)
{
	if (d->disp_Display)
	{
		TImgFreeBitmap(TImgpBase, d->disp_BitMap);
		TDestroy(d->disp_Display);
	}
	TExecFree(TExecBase, d);
}

struct Display *
disp_open(TINT width, TINT height, TBOOL fullscreen)
{
	struct Display *d = TExecAlloc0(TExecBase, TNULL, sizeof(struct Display));
	if (d == TNULL) return TNULL;

	d->disp_Handle.thn_DestroyFunc = (TDFUNC) disp_destroy;

	d->disp_Display = TDisSimpleCreateView(TDispBase, DEFAULT_WINTITLE,
		width, height, 24, TFALSE, fullscreen, TTRUE, TFALSE);

	if (d->disp_Display)
	{
		TTAGITEM attrs[2];
		attrs[0].tti_Tag = TDISTAG_POINTERMODE;
		attrs[0].tti_Value = (TTAG) TDISPTR_INVISIBLE;
		attrs[1].tti_Tag = TTAG_DONE;
		TDisSetAttrs(TDispBase, d->disp_Display, attrs);

		d->disp_BitMap = TImgAllocBitmap(TImgpBase, width, height, IMGFMT_R8G8B8);
		if (d->disp_BitMap)
		{
			d->disp_Width = width;
			d->disp_Height = height;
			d->disp_Depth = 24;
			d->disp_FullScreen = fullscreen;
			return d;
		}
	}

	TDestroy(d);
	return TNULL;
}

TVOID
disp_handleinput(struct Display *d)
{
    TDISMSG msg;
    TDISKEY *key;
    TDISMOUSEPOS *mpos;
	TDISMBUTTON *mb;

	while ((TDisGetMsg(TDispBase, d->disp_Display, &msg)))
	{
		switch(msg.code)
		{
			case TDISMSG_CLOSE:
				d->disp_Flags |= DISPF_ABORT;
				break;

			case TDISMSG_KEYUP:
				key=(TDISKEY*) msg.data;
				if(key->code==TDISKEY_ESCAPE)
					d->disp_Flags |= DISPF_ABORT;
				break;

			case TDISMSG_MOUSEMOVE:
				mpos = (TDISMOUSEPOS *) msg.data;
				d->disp_MouseX = mpos->x;
				d->disp_MouseY = mpos->y;
				break;

			case TDISMSG_MBUTTONDOWN:
				mb = (TDISMBUTTON *) msg.data;
				d->disp_MButton |= mb->code;
				break;

			case TDISMSG_MBUTTONUP:
				mb = (TDISMBUTTON *) msg.data;
				d->disp_MButton &= ~mb->code;
				break;
		}
	}
}

