
#ifndef _TEK_GLOBAL_H
#define _TEK_GLOBAL_H

#include <tek/teklib.h>
#include <tek/proto/exec.h>
#include <tek/proto/util.h>
#include <tek/proto/time.h>
#include <tek/proto/displayhandler.h>
#include <tek/proto/io.h>
#include <tek/proto/imgproc.h>
#include <tek/mod/audio.h>

#ifdef TSYS_DARWIN
	#include <OpenGL/OpenGL.h>
	#include <OpenGL/gl.h>
	#include <OpenGL/glext.h>
	#include <OpenGL/glu.h>
#else
	#include <GL/gl.h>
	#include <GL/glu.h>
#endif

/******************************************************************************/
/*
**	Defaults
*/

#define DEFAULT_MODULE			"../tests/mod.jack"

#define DEFAULT_WIDTH			320
#define DEFAULT_HEIGHT			255
#define DEFAULT_WINTITLE		"PT23"

#define DEFAULT_FORMAT			TADIOFMT_S16M
#define DEFAULT_RATE			44100
//#define DEFAULT_AUDIOBUFSIZE	1024

/******************************************************************************/
/*
**	Argument Template
*/

#define ARG_TEMPLATE	"FILE,-h=HELP/S,-b=BUFSIZE/N,-f=FULLSCREEN/S"
enum { ARG_FILENAME, ARG_HELP, ARG_BUFSIZE, ARG_FULLSCREEN, ARG_NUM };

/******************************************************************************/
/*
**	Global
*/

extern TAPTR TExecBase;
extern TAPTR TUtilBase;
extern TAPTR TTimeBase;
extern TAPTR TImgpBase;
extern TAPTR TDispBase;
extern TAPTR TDthBase;
extern TAPTR TIOBase;

extern TAPTR TimeRequest;
extern TAPTR ArgHandle;
extern TAPTR Arguments[ARG_NUM];

extern TBOOL g_init(TAPTR task);
extern TVOID g_exit(TVOID);

/******************************************************************************/
/*
**	Audio
*/

typedef TCALLBACK TVOID (*PLAYFUNC)(TAPTR audio, TAPTR destbuf);

struct Audio
{
	THNDL aud_Handle;
	TAPTR aud_Task;
	TAPTR aud_UserData;
	struct TAudIORequest *ioreq1, *ioreq2;
	TAPTR aud_PlayLock;
	PLAYFUNC aud_PlayFunc;
	TINT aud_Rate;
	TINT aud_NumSmp;
	TINT aud_NumChn;
	TINT aud_NumBPC;
	TINT aud_NumBPS;
};

extern struct Audio *audio_create(void);
extern TVOID audio_setplayfunc(struct Audio *a, PLAYFUNC f);

/******************************************************************************/
/*
**	Display
*/

struct Display
{
	struct THandle disp_Handle;
	TAPTR disp_Display;
	TINT disp_Width, disp_Height, disp_Depth;
	TBOOL disp_FullScreen;
	TIMGPICTURE *disp_BitMap;
	TUINT disp_Flags;
	TINT disp_MouseX, disp_MouseY;
	TUINT disp_MButton;
};

#define DISPF_ABORT	0x0001

extern struct Display *disp_open(TINT width, TINT height, TBOOL fullscreen);
extern TVOID disp_handleinput(struct Display *d);

#endif
