
#include "tek_global.h"
#include "tek_image.h"

#include <tek/teklib.h>
#include <tek/proto/exec.h>
#include <tek/proto/util.h>
#include <tek/proto/io.h>
#include <tek/proto/imgproc.h>
#include <tek/proto/datatypehandler.h>

TVOID
tFreeImage(tImage *img)
{
	TExecFree(TExecBase, img->pic.palette);
	TExecFree(TExecBase, img->pic.data);
}

/*
**	render image to ARGB
*/

TBOOL
tImage2RGB(tImage *img)
{
	TBOOL success = TTRUE;
	if (img->pic.format != IMGFMT_R8G8B8)
	{
		TIMGPICTURE temp = img->pic;
		success = TFALSE;
		temp.data = TExecAlloc(TExecBase, TNULL, temp.width * temp.height * 3);
		if (temp.data)
		{
			temp.palette = TNULL;
			temp.format = IMGFMT_R8G8B8;
			temp.depth = 24;
			temp.bytesperrow = temp.width * 3;

			if (TImgDoMethod(TImgpBase, &img->pic, &temp, IMGMT_CONVERT, TNULL))
			{
				tFreeImage(img);
				img->pic = temp;
				success = TTRUE;
			}
			else
			{
				TExecFree(TExecBase, temp.data);
			}
		}
	}
	return success;
}


/*
**	scale image to specified width/height, smooth
*/

TBOOL
tScaleImage(tImage *img, TINT w, TINT h)
{
	TBOOL success = TTRUE;
	if (img->pic.width != w || img->pic.width != h)
	{
		TIMGPICTURE temp = img->pic;
		success = TFALSE;
		temp.data = TExecAlloc(TExecBase, TNULL, w * h * sizeof(TUINT));
		if (temp.data)
		{
			TTAGITEM scaletags[2];
			scaletags[0].tti_Tag = IMGTAG_SCALEMETHOD;
			scaletags[0].tti_Value = (TTAG) IMGSMT_SMOOTH;
			scaletags[1].tti_Tag = TTAG_DONE;

			temp.width = w;
			temp.height = h;
			temp.bytesperrow = w * sizeof(TUINT);

			if (TImgDoMethod(TImgpBase, &img->pic, &temp, IMGMT_SCALE, scaletags))
			{
				tFreeImage(img);
				img->pic = temp;
				success = TTRUE;
			}
			else
			{
				TExecFree(TExecBase, temp.data);
			}
		}
	}
	return success;
}


/*
**	load image to an image structure
*/

TBOOL
tLoadImage(tImage *img, TSTRPTR filename)
{
	TBOOL success = TFALSE;
	TAPTR fh;
	fh= io_open(TIOBase, filename, TFMODE_READONLY, TNULL);
	if (fh)
	{
		TAPTR dtobject = TDthOpen(TDthBase, fh);
		if (dtobject)
		{
			TAPTR dttags = TDthGetAttrs(TDthBase, dtobject);
			if ((TINT) TGetTag(dttags, TDTAG_CLASS, 0) == DTCLASS_PIC)
			{
				TIMGPICTURE *pic = &img->pic;

				pic->width = (TINT) TGetTag(dttags, TDTAG_PICWIDTH, 0);
				pic->height = (TINT) TGetTag(dttags, TDTAG_PICHEIGHT, 0);
				pic->depth = (TINT) TGetTag(dttags, TDTAG_PICDEPTH, 0);
				pic->format = (TINT) TGetTag(dttags, TDTAG_PICFORMAT, 0);
				pic->bytesperrow = (TINT) TGetTag(dttags, TDTAG_PICBYTESPERROW, 0);

				pic->data = TExecAlloc(TExecBase, TNULL, pic->bytesperrow * pic->height);
				if (pic->data)
				{
					TTAGITEM loadtags[3];
					loadtags[0].tti_Tag = TDOTAG_GETDATA;
					loadtags[0].tti_Value = (TTAG) pic->data;

					success = TTRUE;
					if (pic->depth <= 8)
					{
						pic->palette = TExecAlloc(TExecBase, TNULL, sizeof(TIMGARGBCOLOR) * 256);
						if (pic->palette)
						{
							loadtags[1].tti_Tag = TDOTAG_GETPALETTE;
							loadtags[1].tti_Value = pic->palette;
							loadtags[2].tti_Tag = TTAG_DONE;
						}
						else
						{
							success = TFALSE;
						}
					}
					else
					{
						pic->palette = TNULL;
						loadtags[1].tti_Tag = TTAG_DONE;
					}

					if (success)
					{
						success = TDthDoMethod(TDthBase, dtobject, loadtags);
					}
				}

				if (success && pic->palette)
				{
					success = tImage2RGB(img);
				}

				if (!success)
				{
					tFreeImage(img);
				}
			}
			TDestroy(dtobject);
		}
		io_close(TIOBase, fh);
	}

	return success;
}


