
#include <stdio.h>
#include "../ptplay/ptplay.h"
#include "tek_global.h"
#include "tek_tga.h"
#include "pt23.h"

/******************************************************************************/
/*
**	callback invoked from the audio thread
*/

static TCALLBACK TVOID
playfunc(struct Audio *a, TUINT8 *buf)
{
	TINT bpc = a->aud_NumBPC;		/* bytes per channel */
	TINT depth = 4 << bpc;			/* bits per channel */

	pt_render(a->aud_UserData,
		buf, buf + bpc, a->aud_NumBPS, a->aud_NumSmp, 1, depth, a->aud_NumChn);
}

/******************************************************************************/
/*
**	mainloop
*/

static TVOID
mainloop(struct Display *d, pt_mod_s *ptmod, pt23_struct *pt)
{
	TTIME t0, t1;
	TTimeQuery(TTimeBase, TimeRequest, &t0);

	do
	{
		disp_handleinput(d);
		pt->mx = d->disp_MouseX;
		pt->my = d->disp_MouseY;
		pt->mb = d->disp_MButton;

		TTimeQuery(TTimeBase, TimeRequest, &t1);
		TTimeSub(TTimeBase, &t1, &t0);
		pt->time = t1.ttm_Sec;

		pt23_drawmain(d->disp_BitMap->data, ptmod, pt);

		if (TDisBegin(TDispBase, d->disp_Display))
		{
			TDisPutImage(TDispBase, d->disp_BitMap, TNULL);
			TDisEnd(TDispBase);
		}

		TDisFlush(TDispBase, d->disp_Display);

	} while (!(d->disp_Flags & DISPF_ABORT));
}

/******************************************************************************/
/*
**	application main
*/

TTASKENTRY TVOID
TEKMain(TAPTR task)
{
	if (g_init(task))
	{
		struct Audio *audio = audio_create();
		if (audio)
		{
			pt_mod_s *ptmod = pt_load(Arguments[ARG_FILENAME], audio->aud_Rate);
			if (ptmod)
			{
				struct Display *disp = disp_open(DEFAULT_WIDTH, DEFAULT_HEIGHT,
					(TBOOL) Arguments[ARG_FULLSCREEN]);

				if (disp)
				{
					pt23_struct *pt = pt23_init();
					if (pt)
					{
						audio->aud_UserData = ptmod;
						audio_setplayfunc(audio, (PLAYFUNC) playfunc);
						mainloop(disp, ptmod, pt);
						pt23_free(pt);
					}
					TDestroy(disp);
				}
				else printf("*** Could not open display\n");
				TDestroy(audio);
			}
			else printf("*** Could not open audio\n");
			pt_free(ptmod);
		}
		else printf("*** Could not load module\n");
		g_exit();
	}
	else printf("*** Not done\n");
}
